/*
 * Copyright (C) 2018 Velocity Contributors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package com.velocitypowered.proxy.network.registry.packet;

import com.velocitypowered.api.network.ProtocolVersion;
import com.velocitypowered.proxy.network.packet.Packet;
import com.velocitypowered.proxy.network.packet.PacketReader;
import io.netty.buffer.ByteBuf;
import org.checkerframework.checker.nullness.qual.Nullable;

public class EmptyPacketRegistryMap implements PacketRegistryMap {

  public static final EmptyPacketRegistryMap INSTANCE = new EmptyPacketRegistryMap();

  private EmptyPacketRegistryMap() {

  }

  @Override
  public @Nullable PacketReader<? extends Packet> lookupReader(final int id,
      ProtocolVersion version) {
    return null;
  }

  @Override
  public <P extends Packet> void writePacket(P packet, ByteBuf buf, ProtocolVersion version) {
    throw new IllegalArgumentException(String.format(
        "Unable to find id for packet of type %s for version %s",
        packet.getClass().getName(), version
    ));
  }

  @Override
  public @Nullable Class<? extends Packet> lookupPacket(int id) {
    return null;
  }
}
